<?php

namespace MR\Zip\Helper;

use Magento\Framework\App\Helper\AbstractHelper;
use Magento\Framework\App\Helper\Context;

class Communication extends AbstractHelper
{
    private $_accessToken;

    private $_date;

    /**
     *
     * @var \MR\Zip\Helper\Configuration
     */
    private $_configuration;

    public function __construct(Context $context, \Magento\Framework\Stdlib\DateTime\DateTime $date)
    {
        parent::__construct($context);
        $objectManager = \Magento\Framework\App\ObjectManager::getInstance();
        $this->_logger = $objectManager->get("\MR\Zip\Logger\ZipLogger");
        $this->_configuration = $objectManager->get("\MR\Zip\Helper\Configuration");
        $this->_logger->info(__METHOD__);
        $this->_accessToken = null;
        $this->_date = $date;
    }

    public function getZipPage($requestData, $storeId = null)
    {
        $this->_logger->info(__METHOD__);

        $orderIncrementId = $requestData['merchantReference'];
        $requestData['merchant']['redirectConfirmUrl'] = $this->_getUrl('zip/order/success', ['_secure' => true, '_nosid' => true, 'mage_order_id' => $orderIncrementId]);
        $requestData['merchant']['redirectCancelUrl'] = $this->_getUrl('zip/order/fail', ['_secure' => true, '_nosid' => true, 'mage_order_id' => $orderIncrementId]);
        $requestData = json_encode($requestData);

        $this->_logger->info(__METHOD__ . " request: ". $requestData);
        $url = $this->_getApiUrl('order', $storeId);
        $header = ['Content-Type: application/json', 'Authorization: Bearer ' . $this->_getAccessToken($storeId)];
        $result = $this->_sendRequest($url, $header, [], \Magento\Framework\HTTP\ZendClient::POST, $requestData);
        return json_decode($result['response'], true);
    }

    public function getTransactionStatus($zipId, $storeId = null)
    {
        $this->_logger->info(__METHOD__ . " zipId:{$zipId} storeId:{$storeId}");

        $zipUrl = $this->_getApiUrl('/order/'. $zipId, $storeId);
        $header = ['Authorization: Bearer ' . $this->_getAccessToken($storeId)];
        $result = $this->_sendRequest($zipUrl, $header);
        return json_decode($result['response'], true);
    }

    public function refund($orderIncrementId, $zipId, $amount, $storeId = null)
    {
        $this->_logger->info(__METHOD__ . "order:{$orderIncrementId} zipId:{$zipId} storeId:{$storeId}");

        $requestData = json_encode([
            'amount'=> $amount,
            'merchantRefundReference' => $orderIncrementId.'-'.$amount.' '.$this->_date->date(),
            ]);
        $this->_logger->info(__METHOD__ . " request: ". $requestData);
        $zipUrl = $this->_getApiUrl('/order/' . $zipId . '/refund/', $storeId);

        $header = ['Content-Type: application/json', 'Authorization: Bearer ' . $this->_getAccessToken($storeId)];
        $result = $this->_sendRequest($zipUrl, $header, [],\Magento\Framework\HTTP\ZendClient::POST, $requestData);
        return $result;
    }

    public function getMerchantConfiguration($storeId)
    {
        $this->_logger->info(__METHOD__ . "storeId:{$storeId}");
        $zipUrl = $this->_getApiUrl('/configuration', $storeId);

        $header = ['Content-Type: application/json', 'Authorization: Bearer ' . $this->_getAccessToken($storeId)];
        $result = $this->_sendRequest($zipUrl, $header);
        return json_decode($result['response'], true);
    }

    protected function _getAccessToken($storeId = null)
    {
        if (!$this->_accessToken) {
            $accessTokenParam = [
                'grant_type' => 'client_credentials',
                'client_id' => $this->_configuration->getZipClientId($storeId),
                'client_secret' => $this->_configuration->getZipClientSecret($storeId),
                'audience' => $this->_configuration->getZipApiAudience($storeId),
            ];

            $headers = [
                'Content-Type: application/json'
            ];
            $url = $this->_configuration->getZipAuthTokenEndpoint($storeId);

            try {
                $accessTokenResult = $this->_sendRequest($url, $headers, [], \Magento\Framework\HTTP\ZendClient::POST, json_encode($accessTokenParam));
                $response = json_decode($accessTokenResult['response'], true);
            } catch (\Exception $ex) {
                $this->_logger->error($ex->getMessage());
                return false;
            }
            if (!$response || !isset($response['access_token'])) {
                $errorMessage = 'Can\'t get Zip AccessToken with the credential.';
                throw new \Magento\Framework\Exception\PaymentException(__($errorMessage));
            }
            $this->_accessToken = $response['access_token'];
        }
        return $this->_accessToken;
    }

    protected function _getApiUrl($path, $storeId = null)
    {
        $baseUrl = $this->_configuration->getZipApiEndpoint($storeId);
        $apiUrl = rtrim($baseUrl, '/') . '/' . trim($path, '/');
        return $apiUrl;
    }

    private function _sendRequest($url, $header = [], $params = [], $method = \Magento\Framework\HTTP\ZendClient::GET, $postBody = null)
    {
        $this->_logger->info(__METHOD__ . " postUrl: {$url}");
        $ch = curl_init();
        switch ($method) {
            case "POST":
                curl_setopt($ch, CURLOPT_POST, 1);

                if ($postBody)
                    curl_setopt($ch, CURLOPT_POSTFIELDS, $postBody);
                break;
            case "PUT":
                curl_setopt($ch, CURLOPT_PUT, 1);
                break;
            default:
                if (!empty($params))
                    $url = sprintf("%s?%s", $url, http_build_query($params));
        }
        curl_setopt($ch, CURLOPT_URL, $url);

        if (!empty($header)) {
            curl_setopt($ch, CURLOPT_HTTPHEADER, $header);
        }

        curl_setopt($ch, CURLOPT_TIMEOUT, 180);
        curl_setopt($ch, CURLOPT_HEADER, false);
        curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);
        curl_setopt($ch, CURLOPT_SSL_VERIFYHOST, false);
        curl_setopt($ch, CURLOPT_SSL_VERIFYPEER, false);

        $response = curl_exec($ch);
        $errorNo = curl_errno($ch);
        $errorMessage = '';
        if($errorNo){
            $errorMessage = " Error:" . curl_error($ch) . " Error Code:" . curl_errno($ch);
            $this->_logger->critical(__METHOD__ . $errorMessage);
        }
        $httpcode = curl_getinfo($ch, CURLINFO_HTTP_CODE);
        if ($httpcode && substr($httpcode, 0, 2) != "20") {
            $errorMessage = " HTTP CODE: {$httpcode} for URL: {$url}";
            $this->_logger->critical(__METHOD__ . $errorMessage);
        }
        $result = [
            'httpcode' => $httpcode,
            'response' => $response,
            'errmsg' => $errorMessage,
        ];
        curl_close($ch);

        $this->_logger->info(__METHOD__ . " response from Zip - HttpCode:{$httpcode} Body:{$response}");
        return $result;
    }
}
